import pytest

from robot.constant import Constant
from robot.finch import Finch
from robot.finch_input import FinchInput
from robot.finch_output import FinchOutput
from robot.hummingbird import Hummingbird
from robot.hummingbird_input import HummingbirdInput
from robot.microbit_output import MicrobitOutput
from robot.request import Request
from robot.state import State


def test_windows_support(mocker):
    mocker.patch.object(Request, "response_from_uri", return_value="200")
    mocker.patch.object(FinchOutput, "wait", return_value=True)

    Request.is_connected('A')

    state = State()

    assert MicrobitOutput.point(state, "A", 3, 3, True)

    hummingbird = Hummingbird('A')

    with pytest.raises(Exception) as e:
        response = hummingbird.light(4)
    assert e.value.message == "Light port 4 out of range"

    with pytest.raises(Exception) as e:
        response = hummingbird.sound(4)
    assert e.value.message == "Sound port 4 out of range"

    with pytest.raises(Exception) as e:
        response = HummingbirdInput.sound("A", 4)
    assert e.value.message == "Sound port 4 out of range"

    assert FinchOutput.move("B", Constant.FORWARD, 7, 5, False)

    with pytest.raises(Exception) as e:
        assert FinchOutput.move("B", "BAD", 4, 5)
    assert e.value.message == "Bad Move Direction: BAD"

    with pytest.raises(Exception) as e:
        assert FinchOutput.turn("B", "BAD", 90, 50)
    assert e.value.message == "Bad Turn Direction: BAD"

    with pytest.raises(Exception) as e:
        FinchInput.encoder("B", "BAD")
    assert e.value.message == "Bad Encoder Side: BAD"


def test_windows_support_finch():
    finch = Finch('B')

    with pytest.raises(Exception) as e:
        finch.encoder("BAD")
    assert e.value.message == "Bad Encoder Side: BAD"



from robot.finch import Finch
from time import sleep

finch = Finch('A')

for i in range(0, 10):
    finch.beak(100, 100, 100)
    sleep(0.1)

    finch.beak(0, 0, 0)
    sleep(0.1)

finch.stop_all()



import pytest
import time

from robot.hummingbird import Hummingbird
from robot.hummingbird_output import HummingbirdOutput


def test_led():
    hummingbird = Hummingbird("A")

    HummingbirdOutput.led(hummingbird.device, 1, 50)
    time.sleep(0.15)

    HummingbirdOutput.led(hummingbird.device, 1, "0")

    with pytest.raises(Exception) as e:
        HummingbirdOutput.led(hummingbird.device, 5, 50)
    assert e.value.message == "Led port 5 out of range"


def test_tri_led():
    hummingbird = Hummingbird("A")

    HummingbirdOutput.tri_led(hummingbird.device, 1, 50, "50", 0)
    time.sleep(0.15)

    HummingbirdOutput.tri_led(hummingbird.device, 1, 0, 0, 0)

    with pytest.raises(Exception) as e:
        HummingbirdOutput.tri_led(hummingbird.device, 5, 50, "50", 0)
    assert e.value.message == "Tri_led port 5 out of range"


def test_position_servo():
    hummingbird = Hummingbird("A")

    HummingbirdOutput.position_servo(hummingbird.device, 1, 20)
    time.sleep(0.15)

    HummingbirdOutput.position_servo(hummingbird.device, 1, 160)
    time.sleep(0.15)


def test_rotation_servo():
    hummingbird = Hummingbird("A")

    HummingbirdOutput.rotation_servo(hummingbird.device, 2, 25)
    time.sleep(0.15)

    HummingbirdOutput.rotation_servo(hummingbird.device, "2", "-25")
    time.sleep(0.15)

    HummingbirdOutput.rotation_servo(hummingbird.device, 2, 0)



import pytest
import time

from robot.constant import Constant
from robot.exception import Exception
from robot.finch_output import FinchOutput
from robot.request import Request


def test_beak():
    assert FinchOutput.beak("B", 10, 50, 50)
    time.sleep(0.15)
    assert FinchOutput.beak("B", 0, 0, 0)


def test_tail():
    assert FinchOutput.tail("B", 1, 10, 50, 50)
    time.sleep(0.1)
    assert FinchOutput.tail("B", 1, "0", 50, "0")
    time.sleep(0.1)
    assert FinchOutput.tail("B", "2", "0", 50, "0")
    time.sleep(0.1)
    assert FinchOutput.tail("B", 3, "0", 50, "0")
    time.sleep(0.1)
    assert FinchOutput.tail("B", 4, "0", 50, "0")
    time.sleep(0.1)
    assert FinchOutput.tail("B", "all", 0, 0, 100)
    time.sleep(0.1)
    assert FinchOutput.tail("B", "all", 0, 0, 0)

    with pytest.raises(Exception) as e:
        assert FinchOutput.tail("B", 5, 10, 50, 50)
    assert e.value.message == "Tail port 5 out of range"


def test_move():
    assert FinchOutput.move("B", Constant.FORWARD, 4, 5)
    assert FinchOutput.move("B", Constant.FORWARD, "4", "5")

    assert FinchOutput.move("B", Constant.BACKWARD, 4, 5)
    assert FinchOutput.move("B", Constant.BACKWARD, "4", "5")

    with pytest.raises(Exception) as e:
        assert FinchOutput.move("B", "BAD", 4, 5)
    assert e.value.message == "Bad Move Direction: BAD"

    with pytest.raises(Exception) as e:
        assert FinchOutput.move("B", None, 4, 5)
    assert e.value.message == "Bad Move Direction: None"

    Request.stop_all("B")


def test_turn():
    assert FinchOutput.turn("B", "L", 25, 50)
    assert FinchOutput.turn("B", "R", 25, 50)
    assert FinchOutput.turn("B", "L", "25", 50)
    assert FinchOutput.turn("B", "R", 25, "50")

    with pytest.raises(Exception) as e:
        assert FinchOutput.turn("B", "BAD", 90, 50)
    assert e.value.message == "Bad Turn Direction: BAD"


def test_motors():
    assert FinchOutput.motors("B", 25, 0)
    time.sleep(0.2)
    assert FinchOutput.motors("B", -25, 0)
    time.sleep(0.2)

    assert FinchOutput.motors("B", 0, -25)
    time.sleep(0.2)
    assert FinchOutput.motors("B", "0", "25")
    time.sleep(0.2)

    Request.stop_all("B")


def test_stop():
    assert FinchOutput.move("B", Constant.FORWARD, 99999, 5, False)
    time.sleep(0.2)
    assert FinchOutput.stop("B")

    assert FinchOutput.move("B", Constant.BACKWARD, 99999, 5, False)
    time.sleep(0.2)
    assert FinchOutput.stop("B")


def test_reset_encoders():
    assert FinchOutput.reset_encoders("B")
